# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# Original framework from Netrender
# Original Octane script from Enrico Cerica

from math import atan, pi, degrees
import subprocess
import os
import sys
import time
import mathutils
import bpy
import octanerender
from octanerender.utils import *

class BUTTON_OCT_export_(bpy.types.Operator):
	bl_idname = "button_export"
	bl_label = "Export OBJ/MTL only"
	bl_description = "Only export OBJ/MTL"

	@classmethod
	def poll(cls, context):
		return True

	def execute(self, context):
		scene = context.scene
		octscene = scene.octane_render

		log ('Exporting OBJ/MTL only')
		octscene.export_mode="export"
		bpy.ops.render.render()
		return{'FINISHED'}

	def invoke(self, context, event):
		return self.execute(context)

class BUTTON_OCT_render(bpy.types.Operator):
	bl_idname = "button_render"
	bl_label = "Render Still Image"
	bl_description = "Export OBJ/MTL, create OCS if it doesn't exist and render"

	@classmethod
	def poll(cls, context):
		return True

	def execute(self, context):
		scene = context.scene
		octscene = scene.octane_render

		log ('Rendering still image')
		octscene.export_mode="still"
		bpy.ops.render.render()
		return{'FINISHED'}

	def invoke(self, context, event):
		return self.execute(context)

class BUTTON_OCT_animation(bpy.types.Operator):
	bl_idname = "button_animation"
	bl_label = "Render Animation"
	bl_description = "Export OBJ/MTL and render animation frame(s)"

	@classmethod
	def poll(cls, context):
		return True

	def execute(self, context):
		scene = context.scene
		octscene = scene.octane_render

		log ('Rendering animation:')
		log ('First frame : %d' % (octscene.animation_start))
		log ('Last  frame : %d' % (octscene.animation_end))
		currentFrame = scene.frame_current
		scene.frame_set(octscene.animation_start)
		#bpy.context.scene.frame_set(octscene.animation_start)

		octscene.export_mode="anim"
		bpy.ops.render.render()

		# Restore the current frame before starting animation
		context.scene.frame_set(currentFrame)				
		return{'FINISHED'}

	def invoke(self, context, event):
		return self.execute(context)

class BUTTON_OCT_select_camera(bpy.types.Operator):
	bl_idname = "button_select_camera"
	bl_label = "Select"
	bl_description = "Select active camera"

	@classmethod
	def poll(cls, context):
		return True

	def execute(self, context):
		scene = context.scene
		octscene = scene.octane_render
		if scene.objects.active.type=="CAMERA":
			octscene.active_camera=scene.objects.active.name
		
		return{'FINISHED'}

	def invoke(self, context, event):
		return self.execute(context)

class button_select_light_source(bpy.types.Operator):
	bl_idname = "button_select_light_source"
	bl_label = "Select"
	bl_description = "Select active light source"

	@classmethod
	def poll(cls, context):
		return True

	def execute(self, context):
		scene = context.scene
		octscene = scene.octane_render
		if scene.objects.active.type=="LAMP":
			octscene.active_light_source=scene.objects.active.name
		
		return{'FINISHED'}

	def invoke(self, context, event):
		return self.execute(context)



