# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

# Original framework from Netrender
# Original Octane script from Enrico Cerica

from math import atan, pi, degrees
import subprocess
import os
import sys
import time
import mathutils
import octanerender

try:
  import bpy
except:
  bpy = None

def log(log):
	if octanerender.Verbose == True:
		print ('Octane plug-in : %s' % (log))

def error(error):
	raise Exception("Octane plug-in: "+error)
# 	self.update_stats("-=|ERROR|", error)
# 
# 	# Hold the render results window while we display the error message
# 	while True:
# 		if self.test_break():
# 			raise Exception("Octane plug-in: "+error)
# 		time.sleep(0.1)

def prefixPath(prefix_directory, file_path, prefix_path, force = False):
    if os.path.isabs(file_path):
        # if an absolute path, make sure path exists, if it doesn't, use relative local path
        full_path = file_path
        if force or not os.path.exists(full_path):
            p, n = os.path.split(os.path.normpath(full_path))

            if prefix_path and p.startswith(prefix_path):
                if len(prefix_path) < len(p):
                    directory = os.path.join(prefix_directory, p[len(prefix_path)+1:]) # +1 to remove separator
                    if not os.path.exists(directory):
                        os.mkdir(directory)
                else:
                    directory = prefix_directory
                full_path = os.path.join(directory, n)
            else:
                full_path = os.path.join(prefix_directory, n)
    else:
        full_path = os.path.join(prefix_directory, file_path)

    return full_path

def matrix_vect(mat,vec):
	vecr = [0.,0.,0.,0.,0.]
	for i in range(4):
		for j in range(4):
			vecr[i] += vec[j] * mat[j][i] 
	return vecr
		
def rotate90x(vect):
	return [vect[0],vect[2],-vect[1]]
	
def fixName(name):
	if name == None:
		return 'None'
	else:
		return name.replace(' ', '_')

def fixPath(path):
	path=str(path)

	if ostype == "linux2" or ostype == "darwin":
		path=path.replace("\\","/")
	#if ostype == "win32":
	else:
		path=path.replace("/","\\")
	return path

def copy_file(source, dest):
	file = open(source, 'rb')
	data = file.read()
	file.close()
	file = open(dest, 'wb')
	file.write(data)
	file.close()

def write_mtl(scene, filepath, copy_images, mtl_dict):

	world = scene.world
	worldAmb = world.ambient_color

	dest_dir = os.path.dirname(filepath)

	def copy_image(image):
		fn = image.filepath
		fn_strip = os.path.basename(fn)
		log (fn_strip)

		if copy_images:
			rel = fn_strip
			fn_abs_dest = os.path.join(dest_dir, fn_strip)
			if not os.path.exists(fn_abs_dest):
				copy_file(fn, fn_abs_dest)
		else:
			rel = fn
		return rel

	file = open(filepath, "w")
	# XXX
	# file.write('# Blender MTL File: %s\n' % Blender.Get('filepath').split('\\')[-1].split('/')[-1])
	file.write('# Material Count: %i\n' % len(mtl_dict))
	# Write material/image combinations we have used.
	for key, (mtl_mat_name, mat, img) in mtl_dict.items():
		# Get the Blender data for the material and the image.
		# Having an image named None will make a bug, dont do it :)
		file.write('newmtl %s\n' % mtl_mat_name) # Define a new material: matname_imgname
		if mat:
			#file.write('Ns %.6f\n' % (mat.specular_hardness*10) ) # Hardness, convert blenders 1-511 to MTL's

			if (mat.use_transparency and mat.transparency_method=="RAYTRACE"):
				#file.write('Ns %.6f\n' % ((1.0-mat.raytrace_transparency.gloss_factor)*10+0.01)) # Roughness
				file.write('Kd %.6f %.6f %.6f\n' % tuple([c for c in mat.diffuse_color]) ) # Diffuse
				file.write('Ka %.6f %.6f %.6f\n' %  tuple([c*mat.ambient for c in worldAmb])  ) # Ambient, uses mirror colour,
				file.write('Ns %.6f\n' % ((mat.specular_hardness-1) * 1.9607843137254901)) # Glossiness - Hardness, convert blenders 1-511 to MTL's
				file.write('Ks %.6f %.6f %.6f\n' % tuple([c for c in mat.specular_color]) ) # Specular
				file.write('Ni %.6f\n' % mat.raytrace_transparency.ior) # Refraction index

			elif (mat.raytrace_mirror.use):
				#file.write('Ns %.6f\n' % ((1.0-mat.raytrace_mirror.gloss_factor)*10+0.01)) # Roughness
				file.write('Kd %.6f %.6f %.6f\n' % tuple([c for c in mat.diffuse_color]) ) # Diffuse
				file.write('Ka %.6f %.6f %.6f\n' %  tuple([c*mat.specular_intensity for c in mat.mirror_color])  ) # Ambient, uses mirror colour,
				file.write('Ns %.6f\n' % ((mat.specular_hardness-1) * 1.9607843137254901)) # Glossiness - Hardness, convert blenders 1-511 to MTL's
				file.write('Ks %.6f %.6f %.6f\n' % tuple([c for c in mat.specular_color]) ) # Specular
				file.write('Ni %.6f\n' % (1+(mat.raytrace_mirror.reflect_factor*7))) # Refraction index

			else:
				file.write('Kd %.6f %.6f %.6f\n' % tuple([c for c in mat.diffuse_color]) ) # Diffuse
				file.write('Ns %.6f\n' % ((mat.specular_hardness-1) * 1.9607843137254901)) # Glossiness - Hardness, convert blenders 1-511 to MTL's
				file.write('Ks %.6f %.6f %.6f\n' % tuple([c for c in mat.specular_color]) ) # Specular

				file.write('d %.6f\n' % mat.alpha) # Alpha (obj uses 'd' for dissolve)
				# 0 to disable lighting, 1 for ambient & diffuse only (specular color set to black), 2 for full lighting.
				if mat.use_shadeless:
					file.write('illum 0\n') # ignore lighting
				elif mat.specular_intensity == 0:
					file.write('illum 1\n') # no specular.
				else:
					file.write('illum 2\n') # light normaly

		else:
			#write a dummy material here?
			file.write('Ns 0\n')
			file.write('Ka %.6f %.6f %.6f\n' %  tuple([c for c in worldAmb])  ) # Ambient, uses mirror colour,
			file.write('Kd 0.8 0.8 0.8\n')
			file.write('Ks 0.8 0.8 0.8\n')
			file.write('d 1\n') # No alpha
			file.write('illum 2\n') # light normaly


		if mat: # No face image. if we have a material search for MTex image.
			for mtex in mat.texture_slots:
				if mtex and mtex.texture.type == 'IMAGE':
					try:
						#filename = mtex.texture.image.filepath.split('\\')[-1].split('/')[-1]
						log ('Source image : %s' % mtex.texture.image.filepath)
						filename=copy_image(mtex.texture.image)
						if mtex.use_map_color_diffuse:
							file.write('map_Kd %s\n' % filename) # Diffuse mapping image
						if mtex.use_map_normal:
							file.write('map_bump %s\n' % filename) # Bump mapping image
						if mtex.use_map_specular:
							file.write('map_Ks %s\n' % filename) # Specular mapping image
						if mtex.use_map_alpha:
							file.write('map_d %s\n' % filename) # Alpha mapping image
						if mtex.use_map_mirror:
							file.write('map_Ka %s\n' % filename) # Mirror mapping image
						if mtex.use_map_hardness:
							file.write('map_Ns %s\n' % filename) # Hardness to affect roughness in octane
					except:
						# Texture has no image though its an image type, best ignore.
						log ("No image")
		file.write('\n\n')
	file.close()
    
def test_nurbs_compat(ob):
    if ob.type != 'Curve':
        return False

    for nu in ob.data:
        if (not nu.knotsV) and nu.type != 1: # not a surface and not bezier
            return True

    return False


# XXX not converted
def write_nurb(file, ob, ob_mat):
    tot_verts = 0
    cu = ob.data

    # use negative indices
    Vector = Blender.mathutils.Vector
    for nu in cu:

        if nu.type==0:          DEG_ORDER_U = 1
        else:                           DEG_ORDER_U = nu.orderU-1  # Tested to be correct

        if nu.type==1:
            log('\tWarning, bezier curve: %' % (ob.name) % ' only poly and nurbs curves supported')
            continue

        if nu.knotsV:
            log('\tWarning, surface: %s' % (ob.name) % ' only poly and nurbs curves supported')
            continue

        if len(nu) <= DEG_ORDER_U:
            log('\tWarning, orderU is lower then vert count, skipping: %s' % (ob.name))
            continue

        pt_num = 0
        do_closed = (nu.flagU & 1)
        do_endpoints = (do_closed==0) and (nu.flagU & 2)

        for pt in nu:
            pt = Vector(pt[0], pt[1], pt[2]) * ob_mat
            file.write('v %.6f %.6f %.6f\n' % (pt[0], pt[1], pt[2]))
            pt_num += 1
        tot_verts += pt_num

        file.write('g %s\n' % (fixName(ob.name))) # fixName(ob.getData(1)) could use the data name too
        file.write('cstype bspline\n') # not ideal, hard coded
        file.write('deg %d\n' % DEG_ORDER_U) # not used for curves but most files have it still

        curve_ls = [-(i+1) for i in range(pt_num)]

        # 'curv' keyword
        if do_closed:
            if DEG_ORDER_U == 1:
                pt_num += 1
                curve_ls.append(-1)
            else:
                pt_num += DEG_ORDER_U
                curve_ls = curve_ls + curve_ls[0:DEG_ORDER_U]

        file.write('curv 0.0 1.0 %s\n' % (' '.join( [str(i) for i in curve_ls] ))) # Blender has no U and V values for the curve

        # 'parm' keyword
        tot_parm = (DEG_ORDER_U + 1) + pt_num
        tot_parm_div = float(tot_parm-1)
        parm_ls = [(i/tot_parm_div) for i in range(tot_parm)]

        if do_endpoints: # end points, force param
            for i in range(DEG_ORDER_U+1):
                parm_ls[i] = 0.0
                parm_ls[-(1+i)] = 1.0

        file.write('parm u %s\n' % ' '.join( [str(i) for i in parm_ls] ))

        file.write('end\n')

    return tot_verts

def write(filepath, objects, scene,
          EXPORT_TRI=False,
          EXPORT_EDGES=False,
          EXPORT_NORMALS=False,
          EXPORT_NORMALS_HQ=False,
          EXPORT_UV=True,
          EXPORT_MTL=True,
          EXPORT_COPY_IMAGES=False,
          EXPORT_APPLY_MODIFIERS=True,
          EXPORT_ROTX90=True,
          EXPORT_BLEN_OBS=True,
          EXPORT_GROUP_BY_OB=False,
          EXPORT_GROUP_BY_MAT=False,
          EXPORT_KEEP_VERT_ORDER=False,
          EXPORT_POLYGROUPS=False,
          EXPORT_CURVE_AS_NURBS=True):
    '''
    Basic write function. The context and options must be alredy set
    This can be accessed externaly
    eg.
    write( 'c:\\test\\foobar.obj', Blender.Object.GetSelected() ) # Using default options.
    '''
    octscene = scene.octane_render
    # XXX
    import math

    unitFactor = 1
	#Set the unit factor
    unitFactor = {
        0:0.001,
        1:0.01,
        2:0.1,
        3:1,
        4:10,
        5:100,
        6:1000,
        7:0.0254,
        8:0.3048,
        9:0.9144,
        10:201.168,
        11:1609.344               
    }[int(octscene.unit_size)]

    def veckey3d(v):
        return round(v.x, 6), round(v.y, 6), round(v.z, 6)

    def veckey2d(v):
        return round(v[0], 6), round(v[1], 6)
        # return round(v.x, 6), round(v.y, 6)

    def findVertexGroupName(face, vWeightMap):
        """
        Searches the vertexDict to see what groups is assigned to a given face.
        We use a frequency system in order to sort out the name because a given vetex can
        belong to two or more groups at the same time. To find the right name for the face
        we list all the possible vertex group names with their frequency and then sort by
        frequency in descend order. The top element is the one shared by the highest number
        of vertices is the face's group
        """
        weightDict = {}
        for vert_index in face.vertices:
            vWeights = vWeightMap[vert_index]
            for vGroupName, weight in vWeights:
                weightDict[vGroupName] = weightDict.get(vGroupName, 0) + weight

        if weightDict:
            alist = [(weight,vGroupName) for vGroupName, weight in weightDict.items()] # sort least to greatest amount of weight
            alist.sort()
            return(alist[-1][1]) # highest value last
        else:
            return '(null)'

    # TODO: implement this in C? dunno how it should be called...
    def getVertsFromGroup(me, group_index):
        ret = []

        for i, v in enumerate(me.vertices):
            for g in v.groups:
                if g.group == group_index:
                    ret.append((i, g.weight))

        return ret


    log('OBJ Export path: "%s"' % filepath)
    temp_mesh_name = '~tmp-mesh'
    time1 = time.clock()

    file = open(filepath, "w")
    # Write Header
    file.write('# Blender v%s OBJ File: %s\n' % (bpy.app.version_string, bpy.data.filepath.split('/')[-1].split('\\')[-1] ))
    file.write('# www.blender.org\n')

    # Tell the obj file what material file to use.
    if EXPORT_MTL:
        mtlfilepath = '%s.mtl' % '.'.join(filepath.split('.')[:-1])
        file.write('mtllib %s\n' % ( mtlfilepath.split('\\')[-1].split('/')[-1] ))

    if EXPORT_ROTX90:
        mat_xrot90= mathutils.Matrix.Rotation(-math.pi/2, 4, 'X')

    # Initialize totals, these are updated each object
    totverts = totuvco = totno = 1

    face_vert_index = 1

    globalNormals = {}

    # A Dict of Materials
    # (material.name, image.name):matname_imagename # matname_imagename has gaps removed.
    mtl_dict = {}

    # Get all meshes
    for ob_main in objects:
#        if ob_main.parent and ob_main.parent.duplis_used:
#          log(ob_main.type % 'duplis ')
        #,ob_main.parent.dupli_type)
        # ignore dupli children
        if ob_main.parent and ob_main.parent.dupli_type != 'NONE':
            # XXX
            log(ob_main.name % ' is a dupli child - ignoring')
            continue

        obs = []
        if ob_main.dupli_type != 'NONE':
            # XXX
            log('creating dupli_list on %s' % ob_main.name)
            ob_main.create_dupli_list(scene)

            obs = [(dob.object, dob.matrix) for dob in ob_main.dupli_list]

            # XXX debug log
            log(ob_main.name % 'has %d' % (len(obs)) % 'dupli children')
        else:
            obs = [(ob_main, ob_main.matrix_world)]

        for ob, ob_mat in obs:

            # Nurbs curve support
            if EXPORT_CURVE_AS_NURBS and test_nurbs_compat(ob):
                if EXPORT_ROTX90:
                   ob_mat = ob_mat * mat_xrot90
                totverts += write_nurb(file, ob, ob_mat)
                continue
            # END NURBS

#             if ob.type != 'MESH':
#                 continue

            me = ob.create_mesh(scene, EXPORT_APPLY_MODIFIERS, 'RENDER')
#             me = ob.create_mesh(scene, EXPORT_APPLY_MODIFIERS, 'PREVIEW')

#                       # Will work for non meshes now! :)
#            me= BPyMesh.getMeshFromObject(ob, none, True, True, scene)
            if not me:
                continue

            if EXPORT_ROTX90:
                me.transform(mat_xrot90 * ob_mat)
            else:
                me.transform(ob_mat)


            if EXPORT_UV:
                faceuv = len(me.uv_textures) > 0
                if faceuv:
                    uv_layer = me.uv_textures.active.data[:]
            else:
                faceuv = False

            me_verts = me.vertices[:]

            # Make our own list so it can be sorted to reduce context switching
            face_index_pairs = [ (face, index) for index, face in enumerate(me.faces)]
            # faces = [ f for f in me.faces ]

            if EXPORT_EDGES:
                edges = me.edges
            else:
                edges = []

            if not (len(face_index_pairs)+len(edges)+len(me.vertices)): # Make sure there is somthing to write

                # clean up
                bpy.data.meshes.remove(me)

                continue # dont bother with this mesh.

            # XXX
            # High Quality Normals
            if EXPORT_NORMALS and face_index_pairs:
                me.calc_normals()
#                               if EXPORT_NORMALS_HQ:
#                                       BPyMesh.meshCalcNormals(me)
#                               else:
#                                       # transforming normals is incorrect
#                                       # when the matrix is scaled,
#                                       # better to recalculate them
#                                       me.calcNormals()

			#materials = me.getMaterials(1) # 1 == will return None in the list.
            if ob.type == "TEXT" or ob.type == "CURVE":
                materials = ob.data.materials
            else:
                materials = me.materials

            materialNames = []
            materialItems = [m for m in materials]
            if materials:
                for mat in materials:
                    if mat: # !=None
                        materialNames.append(mat.name)
                    else:
                        materialNames.append(None)
                # Cant use LC because some materials are None.
                # materialNames = map(lambda mat: mat.name, materials) # Bug Blender, dosent account for null materials, still broken.

            # Possible there null materials, will mess up indicies
            # but at least it will export, wait until Blender gets fixed.
            materialNames.extend((16-len(materialNames)) * [None])
            materialItems.extend((16-len(materialItems)) * [None])

            # Sort by Material, then images
            # so we dont over context switch in the obj file.
            if EXPORT_KEEP_VERT_ORDER:
                pass
            elif faceuv:
                # XXX update
                tface = me.uv_textures.active.data

                face_index_pairs.sort(key=lambda a: (a[0].material_index, hash(tface[a[1]].image), a[0].use_smooth))
            elif len(materials) > 1:
                face_index_pairs.sort(key = lambda a: (a[0].material_index, a[0].use_smooth))
            else:
                # no materials
                face_index_pairs.sort(key = lambda a: a[0].use_smooth)
#                       if EXPORT_KEEP_VERT_ORDER:
#                               pass
#                       elif faceuv:
#                               try:    faces.sort(key = lambda a: (a.mat, a.image, a.use_smooth))
#                               except: faces.sort(lambda a,b: cmp((a.mat, a.image, a.use_smooth), (b.mat, b.image, b.use_smooth)))
#                       elif len(materials) > 1:
#                               try:    faces.sort(key = lambda a: (a.mat, a.use_smooth))
#                               except: faces.sort(lambda a,b: cmp((a.mat, a.use_smooth), (b.mat, b.use_smooth)))
#                       else:
#                               # no materials
#                               try:    faces.sort(key = lambda a: a.use_smooth)
#                               except: faces.sort(lambda a,b: cmp(a.use_smooth, b.use_smooth))

            faces = [pair[0] for pair in face_index_pairs]

            # Set the default mat to no material and no image.
            contextMat = (0, 0) # Can never be this, so we will label a new material teh first chance we get.
            contextSmooth = None # Will either be true or false,  set bad to force initialization switch.

            if EXPORT_BLEN_OBS or EXPORT_GROUP_BY_OB:
                name1 = ob.name
                name2 = ob.data.name
                if name1 == name2:
                    obnamestring = fixName(name1)
                else:
                    obnamestring = '%s_%s' % (fixName(name1), fixName(name2))

                if EXPORT_BLEN_OBS:
                    file.write('o %s\n' % obnamestring) # Write Object name
                else: # if EXPORT_GROUP_BY_OB:
                    file.write('g %s\n' % obnamestring)


            # Vert
            for v in me.vertices:
                file.write('v %.6f %.6f %.6f\n' % tuple(v.co*unitFactor))

            # UV
            if faceuv:
                uv_face_mapping = [[0,0,0,0] for i in range(len(face_index_pairs))] # a bit of a waste for tri's :/

                uv_dict = {} # could use a set() here
                uv_layer = me.uv_textures.active.data
                for f, f_index in face_index_pairs:
                    for uv_index, uv in enumerate(uv_layer[f_index].uv):
                        uvkey = veckey2d(uv)
                        try:
                            uv_face_mapping[f_index][uv_index] = uv_dict[uvkey]
                        except:
                            uv_face_mapping[f_index][uv_index] = uv_dict[uvkey] = len(uv_dict)
                            file.write('vt %.6f %.6f\n' % tuple(uv))

                uv_unique_count = len(uv_dict)
#               del uv, uvkey, uv_dict, f_index, uv_index
                # Only need uv_unique_count and uv_face_mapping

            # NORMAL, Smooth/Non smoothed.
            if EXPORT_NORMALS:
                for f, f_index in face_index_pairs:
                    if f.use_smooth:
                        for v_idx in f.vertices:
                            v = me_verts[v_idx]
                            noKey = veckey3d(v.normal)
                            if noKey not in globalNormals:
                                globalNormals[noKey] = totno
                                totno +=1
                                file.write('vn %.6f %.6f %.6f\n' % noKey)
                    else:
                        # Hard, 1 normal from the face.
                        noKey = veckey3d(f.normal)
                        if noKey not in globalNormals:
                            globalNormals[noKey] = totno
                            totno +=1
                            file.write('vn %.6f %.6f %.6f\n' % noKey)

            if not faceuv:
                f_image = None

            # XXX
            if EXPORT_POLYGROUPS:
                # Retrieve the list of vertex groups
#                               vertGroupNames = me.getVertGroupNames()

                currentVGroup = ''
                # Create a dictionary keyed by face id and listing, for each vertex, the vertex groups it belongs to
                vgroupsMap = [[] for _i in range(len(me.vertices))]
#                               vgroupsMap = [[] for _i in xrange(len(me.verts))]
                for g in ob.vertex_groups:
#                               for vertexGroupName in vertGroupNames:
                    for vIdx, vWeight in getVertsFromGroup(me, g.index):
#                                       for vIdx, vWeight in me.getVertsFromGroup(vertexGroupName, 1):
                        vgroupsMap[vIdx].append((g.name, vWeight))

            for f_index, f in enumerate(faces):
                f_v = [{"index": index, "vertex": me.vertices[index]} for index in f.vertices]

                # if f.verts[3] == 0:
                #       f_v.pop()

#                               f_v= f.v
                f_smooth= f.use_smooth
                f_mat = min(f.material_index, len(materialNames)-1)
#                               f_mat = min(f.mat, len(materialNames)-1)
                if faceuv:

                    tface = me.uv_textures.active.data[face_index_pairs[f_index][1]]

                    f_image = tface.image
                    f_uv = tface.uv
                    # f_uv= [tface.uv1, tface.uv2, tface.uv3]
                    # if len(f.verts) == 4:
                    #   f_uv.append(tface.uv4)
#                                       f_image = f.image
#                                       f_uv= f.uv

                # MAKE KEY
                if faceuv and f_image: # Object is always true.
                    key = materialNames[f_mat],  f_image.name
                else:
                    key = materialNames[f_mat],  None # No image, use None instead.

                # Write the vertex group
                if EXPORT_POLYGROUPS:
                    if len(ob.vertex_groups):
                        # find what vertext group the face belongs to
                        theVGroup = findVertexGroupName(f,vgroupsMap)
                        if      theVGroup != currentVGroup:
                            currentVGroup = theVGroup
                            file.write('g %s\n' % theVGroup)
#                               # Write the vertex group
#                               if EXPORT_POLYGROUPS:
#                                       if vertGroupNames:
#                                               # find what vertext group the face belongs to
#                                               theVGroup = findVertexGroupName(f,vgroupsMap)
#                                               if      theVGroup != currentVGroup:
#                                                       currentVGroup = theVGroup
#                                                       file.write('g %s\n' % theVGroup)

                # CHECK FOR CONTEXT SWITCH
                if key == contextMat:
                    pass # Context alredy switched, dont do anything
                else:
                    if key[0] == None and key[1] == None:
                        # Write a null material, since we know the context has changed.
                        if EXPORT_GROUP_BY_MAT:
                            # can be mat_image or (null)
                            file.write('g %s_%s\n' % (fixName(ob.name), fixName(ob.data.name)) ) # can be mat_image or (null)
                        file.write('usemtl (null)\n') # mat, image

                    else:
                        mat_data= mtl_dict.get(key)
                        if not mat_data:
                            # First add to global dict so we can export to mtl
                            # Then write mtl

                            # Make a new names from the mat and image name,
                            # converting any spaces to underscores with fixName.

                            # If none image dont bother adding it to the name
#                            if key[1] == None:
                                mat_data = mtl_dict[key] = ('%s'%fixName(key[0])), materialItems[f_mat], f_image
 #                           else:
  #                              mat_data = mtl_dict[key] = ('%s_%s' % (fixName(key[0]), fixName(key[1]))), materialItems[f_mat], f_image

                        if EXPORT_GROUP_BY_MAT:
                            file.write('g %s_%s_%s\n' % (fixName(ob.name), fixName(ob.data.name), mat_data[0]) ) # can be mat_image or (null)

                        file.write('usemtl %s\n' % mat_data[0]) # can be mat_image or (null)

                contextMat = key
                if f_smooth != contextSmooth:
                    if f_smooth: # on now off
                        file.write('s 1\n')
                        contextSmooth = f_smooth
                    else: # was off now on
                        file.write('s off\n')
                        contextSmooth = f_smooth

                file.write('f')
                if faceuv:
                    if EXPORT_NORMALS:
                        if f_smooth: # Smoothed, use vertex normals
                            for vi, v in enumerate(f_v):
                                file.write( ' %d/%d/%d' % \
                                                (v["index"] + totverts,
                                                 totuvco + uv_face_mapping[f_index][vi],
                                                 globalNormals[ veckey3d(v["vertex"].normal) ]) ) # vert, uv, normal

                        else: # No smoothing, face normals
                            no = globalNormals[ veckey3d(f.normal) ]
                            for vi, v in enumerate(f_v):
                                file.write( ' %d/%d/%d' % \
                                                (v["index"] + totverts,
                                                 totuvco + uv_face_mapping[f_index][vi],
                                                 no) ) # vert, uv, normal
                    else: # No Normals
                        for vi, v in enumerate(f_v):
                            file.write( ' %d/%d' % (\
                              v["index"] + totverts,\
                              totuvco + uv_face_mapping[f_index][vi])) # vert, uv

                    face_vert_index += len(f_v)

                else: # No UV's
                    if EXPORT_NORMALS:
                        if f_smooth: # Smoothed, use vertex normals
                            for v in f_v:
                                file.write( ' %d//%d' %
                                            (v["index"] + totverts, globalNormals[ veckey3d(v["vertex"].normal) ]) )
                        else: # No smoothing, face normals
                            no = globalNormals[ veckey3d(f.normal) ]
                            for v in f_v:
                                file.write( ' %d//%d' % (v["index"] + totverts, no) )
                    else: # No Normals
                        for v in f_v:
                            file.write( ' %d' % (v["index"] + totverts) )

                file.write('\n')

            # Write edges.
            if EXPORT_EDGES:
                for ed in edges:
                    if ed.is_loose:
                        file.write('f %d %d\n' % (ed.vertices[0] + totverts, ed.vertices[1] + totverts))

            # Make the indicies global rather then per mesh
            totverts += len(me.vertices)
            if faceuv:
                totuvco += uv_unique_count

            # clean up
            bpy.data.meshes.remove(me)

        if ob_main.dupli_type != 'NONE':
            ob_main.free_dupli_list()

    file.close()


    # Now we have all our materials, save them
    if EXPORT_MTL:
        write_mtl(scene, mtlfilepath, octscene.export_copy_images, mtl_dict)
#       if EXPORT_COPY_IMAGES:
#               dest_dir = os.path.basename(filepath)
# #             dest_dir = filepath
# #             # Remove chars until we are just the path.
# #             while dest_dir and dest_dir[-1] not in '\\/':
# #                     dest_dir = dest_dir[:-1]
#               if dest_dir:
#                       copy_images(dest_dir, mtl_dict)
#               else:
#                       log('\tError: "%s" could not be used as a base for an image path.' % filepath)

    log('OBJ Export time: %.2f' % (time.clock() - time1))

